@@ -1,14 +1,18 @@
 // SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause
 /*
- * Copyright 2018-2023 Amazon.com, Inc. or its affiliates. All rights reserved.
+ * Copyright 2018-2024 Amazon.com, Inc. or its affiliates. All rights reserved.
  */
 
+#include "kcompat.h"
 #include <linux/module.h>
 #include <linux/pci.h>
 #include <linux/utsname.h>
 #include <linux/version.h>
 
 #include <rdma/ib_user_verbs.h>
+#ifdef HAVE_IB_DEVICE_DRIVER_DEF
+#include <rdma/uverbs_ioctl.h>
+#endif
 
 #include "efa.h"
 #include "efa_sysfs.h"
@@ -32,7 +36,7 @@ static const struct pci_device_id efa_pci_tbl[] = {
 };
 
 #define DRV_MODULE_VER_MAJOR           2
-#define DRV_MODULE_VER_MINOR           6
+#define DRV_MODULE_VER_MINOR           8
 #define DRV_MODULE_VER_SUBMINOR        0
 
 #ifndef DRV_MODULE_VERSION
@@ -60,6 +64,10 @@ MODULE_DEVICE_TABLE(pci, efa_pci_tbl);
 	(BIT(EFA_ADMIN_FATAL_ERROR) | BIT(EFA_ADMIN_WARNING) | \
 	 BIT(EFA_ADMIN_NOTIFICATION) | BIT(EFA_ADMIN_KEEP_ALIVE))
 
+#ifdef HAVE_IB_DEVICE_DRIVER_DEF
+extern const struct uapi_definition efa_uapi_defs[];
+#endif
+
 /* This handler will called for unknown event group or unimplemented handlers */
 static void unimplemented_aenq_handler(void *data,
 				       struct efa_admin_aenq_entry *aenq_e)
@@ -311,6 +319,8 @@ static void efa_set_host_info(struct efa_dev *dev)
 		EFA_COMMON_SPEC_VERSION_MINOR);
 #ifdef HAVE_EFA_P2P
 	EFA_SET(&hinf->flags, EFA_ADMIN_HOST_INFO_GDR, 1);
+#else
+	EFA_SET(&hinf->flags, EFA_ADMIN_HOST_INFO_GDR, 0);
 #endif
 
 	efa_com_set_feature_ex(&dev->edev, &resp, &cmd, EFA_ADMIN_HOST_INFO,
@@ -590,6 +600,10 @@ static int efa_ib_device_add(struct efa_dev *dev)
 	dev->ibdev.req_notify_cq = efa_req_notify_cq;
 #endif
 
+#ifdef HAVE_IB_DEVICE_DRIVER_DEF
+	dev->ibdev.driver_def = efa_uapi_defs;
+#endif
+
 #ifdef HAVE_IB_REGISTER_DEVICE_DMA_DEVICE_PARAM
 	err = ib_register_device(&dev->ibdev, "efa_%d", &pdev->dev);
 #elif defined(HAVE_IB_REGISTER_DEVICE_TWO_PARAMS)
