@@ -3,15 +3,17 @@
  * Copyright 2015-2022 Amazon.com, Inc. or its affiliates. All rights reserved.
  */
 
-#include "ena_devlink.h"
+#include <linux/pci.h>
+#include "ena_netdev.h"
 #include "ena_phc.h"
 
 #ifdef ENA_PHC_SUPPORT
-
+#ifdef ENA_PHC_SUPPORT_ADJFREQ
 static int ena_phc_adjfreq(struct ptp_clock_info *clock_info, s32 ppb)
 {
 	return -EOPNOTSUPP;
 }
+#endif /* ENA_PHC_SUPPORT_ADJFREQ */
 
 static int ena_phc_adjtime(struct ptp_clock_info *clock_info, s64 delta)
 {
@@ -38,7 +40,7 @@ static int ena_phc_gettimex64(struct ptp_clock_info *clock_info, struct timespec
 
 	ptp_read_system_prets(sts);
 
-	rc = ena_com_phc_get(phc_info->adapter->ena_dev, &timestamp_nsec);
+	rc = ena_com_phc_get_timestamp(phc_info->adapter->ena_dev, &timestamp_nsec);
 
 	ptp_read_system_postts(sts);
 
@@ -59,7 +61,7 @@ static int ena_phc_gettime64(struct ptp_clock_info *clock_info, struct timespec6
 
 	spin_lock_irqsave(&phc_info->lock, flags);
 
-	rc = ena_com_phc_get(phc_info->adapter->ena_dev, &timestamp_nsec);
+	rc = ena_com_phc_get_timestamp(phc_info->adapter->ena_dev, &timestamp_nsec);
 
 	spin_unlock_irqrestore(&phc_info->lock, flags);
 
@@ -86,7 +88,7 @@ static int ena_phc_gettime(struct ptp_clock_info *clock_info, struct timespec *t
 
 	spin_lock_irqsave(&phc_info->lock, flags);
 
-	rc = ena_com_phc_get(phc_info->adapter->ena_dev, &timestamp_nsec);
+	rc = ena_com_phc_get_timestamp(phc_info->adapter->ena_dev, &timestamp_nsec);
 
 	spin_unlock_irqrestore(&phc_info->lock, flags);
 
@@ -109,7 +111,9 @@ static struct ptp_clock_info ena_ptp_clock_info = {
 	.n_ext_ts	= 0,
 	.n_per_out	= 0,
 	.pps		= 0,
+#ifdef ENA_PHC_SUPPORT_ADJFREQ
 	.adjfreq	= ena_phc_adjfreq,
+#endif /* ENA_PHC_SUPPORT_ADJFREQ */
 	.adjtime	= ena_phc_adjtime,
 #ifdef ENA_PHC_SUPPORT_GETTIME64
 #ifdef ENA_PHC_SUPPORT_GETTIME64_EXTENDED
@@ -164,6 +168,10 @@ static int ena_phc_register(struct ena_adapter *adapter)
 	phc_info = adapter->phc_info;
 	clock_info = &phc_info->clock_info;
 
+	/* PHC may already be registered in case of a reset */
+	if (ena_phc_is_active(adapter))
+		return 0;
+
 	phc_info->adapter = adapter;
 
 	spin_lock_init(&phc_info->lock);
@@ -189,7 +197,8 @@ static void ena_phc_unregister(struct ena_adapter *adapter)
 {
 	struct ena_phc_info *phc_info = adapter->phc_info;
 
-	if (ena_phc_is_active(adapter)) {
+	/* During reset flow, PHC must stay registered to keep kernel's PHC index */
+	if (ena_phc_is_active(adapter) && !test_bit(ENA_FLAG_TRIGGER_RESET, &adapter->flags)) {
 		ptp_clock_unregister(phc_info->clock);
 		phc_info->clock = NULL;
 	}
@@ -260,7 +269,6 @@ err_ena_com_phc_config:
 	ena_com_phc_destroy(ena_dev);
 err_ena_com_phc_init:
 	ena_phc_enable(adapter, false);
-	ena_devlink_disable_phc_param(adapter->devlink);
 	return rc;
 }
 
@@ -278,4 +286,11 @@ int ena_phc_get_index(struct ena_adapter *adapter)
 	return -1;
 }
 
+int ena_phc_get_error_bound(struct ena_adapter *adapter, u32 *error_bound_nsec)
+{
+	if (!ena_phc_is_active(adapter))
+		return -EOPNOTSUPP;
+
+	return ena_com_phc_get_error_bound(adapter->ena_dev, error_bound_nsec);
+}
 #endif /* ENA_PHC_SUPPORT */
